#!/bin/sh
# date-order-arg.sh -- sets DATE_ORDER to argument to sort commits by date
# This script is meant to be sourced from other scripts in this directory

# Copyright 2012-2016 University of New Hampshire Interoperability Laboratory
# All rights reserved.
#
# This file is part of the UNH EXS implementation of the
# Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
# Document Number C050 published by The Open Group, January 2005,
# and copyright 2005 by The Open Group.
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available from the file
# COPYING.GPL in the main directory of this source tree, or the
# BSD license, available from the file COPYING.BSD in the main directory
# of this source tree.
#
# This material is based upon work supported by the National Science
# Foundation under Grants No. OCI-1127228 and DGE-0913620.
#
# Any opinions, findings, and conclusions or recommendations
# expressed in this material are those of the authors and do not
# necessarily reflect the views of the National Science Foundation.

# git keeps two timestamps for each commit: when the commit was authored and
# when the commit object itself was created.  The first will not change during
# rebase, push/pull, etc., but the second will.  When tracking modification
# dates, we want to use the author date as our best approximation, but this
# requires a recent git.  Fall back to ordering by commit date if author date
# isn't present, but loudly warn since this will produce incorrect information
# in the resulting tarball.

if git log -n 1 --author-date-order >/dev/null 2>&1; then
	DATE_ORDER=--author-date-order
else
	echo
	echo WARNING: Your git does not support --author-date-order
	echo WARNING: Dates in man pages may be incorrect as a result
	echo WARNING: Please upgrade your git to at least 1.8.4
	echo
	DATE_ORDER=--date-order
fi
