/*
 * Copyright 2007-2017 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#include "result.h"
#include "setup/exs_init_setup.h"
#include "setup/exs_socket_setup.h"
#include "setup/exs_connect_setup.h"
#include "setup/exs_bind_setup.h"
#include "setup/exs_listen_setup.h"
#include "setup/exs_accept_setup.h"
#include "setup/exs_client_setup.h"
#include "setup/exs_server_setup.h"

#define BAD_FLAG 2		//TODO

#define BLOCKING_CLOSE_TESTS 2

static char *all_blocking_close_tests[BLOCKING_CLOSE_TESTS] = {
	"exs_blocking_close(fd) [no exs_init]",
	"exs_blocking_close(fd)",
};

static void *test_blocking_close(void *test_void_ptr)
{
	param *p = (param *) test_void_ptr;
	int test = p->test_number;

	if (p->mode > 0)
		return 0;

	int fd;

	/* EPERM: exs_init has not been called */
	if (test == 1) {
		p->test_value = "EPERM";
		p->test_name = all_blocking_close_tests[test - 1];
		setup_fault_catcher(p, 0);
		int ret = exs_blocking_close(p->fd);

		if (ret == 0) {
			print_macro(test, p->test_value, p->test_name, ret,
				    0, 0);
		} else {
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, EPERM);
		}
	}

	/* OK_EXS_BLOCKING_CLOSE */
	else if (test == 2) {
		pid_t pid = fork();
		if (pid == 0) {
			exs_client(p);
			exit(EXIT_SUCCESS);
		} else {
			p->test_value = "OK_EXS_CLOSE";
			p->test_name = all_blocking_close_tests[test - 1];
			setup_fault_catcher(p, pid);

			if (p->mode > 0) {
				exs_server(p);
			}

			if (p->mode == 0) {
				if (ok_exs_init(p) < 0)
					exit(EXIT_FAILURE);

				if (ok_exs_socket(p) < 0)
					exit(EXIT_FAILURE);
				if (ok_exs_bind(p) < 0)
					exit(EXIT_FAILURE);
				if (ok_exs_listen(p) < 0)
					exit(EXIT_FAILURE);

				fd = open(exs_test_fifo_1, O_WRONLY);

				if (ok_exs_accept(p) < 0)
					exit(EXIT_FAILURE);

				close(fd);
			}

			int ret = exs_blocking_close(p->fd);

			if (ret == 0) {
				print_macro(test, p->test_value,
					    p->test_name, ret, 0, 0);
			} else {
				print_macro(test, p->test_value,
					    p->test_name, ret, -1, EINVAL);
			}

			if (p->mode == 0) {
				fd = open(exs_test_fifo_2, O_RDONLY);
				close(fd);

				exit(EXIT_SUCCESS);
			}
		}
	}

	/** test number not valid **/
	else if (test != 0) {
		fprintf(stdout, "Invalid Test Number: %d\n", test);
	}

	return NULL;
}

int main(int argc, char *argv[])
{
	execute(argc, argv, test_blocking_close, all_blocking_close_tests, BLOCKING_CLOSE_TESTS);
}
