/*
 * Copyright 2007-2017 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#include "result.h"
#include "setup/exs_init_setup.h"
#include "setup/exs_mregister_setup.h"
#include "setup/exs_mderegister_setup.h"

#define BAD_FLAG 2		//TODO

#define MDEREGISTER_TESTS 6

static char *all_mderegister_tests[MDEREGISTER_TESTS] = {
	"exs_mderegister(mh, 0) [no exs_init]",
	"exs_mderegister(NULL, 0)",
	"exs_mderegister(EXS_MHANDLE_INVALID, 0)",
	"exs_mderegister(EXS_MHANDLE_UNREGISTERED, 0)",
	"exs_mderegister(mh, BAD_FLAG)",
	"exs_mderegister(mh, 0)"
};

static void *test_mderegister(void *test_void_ptr)
{
	param *p = (param *) test_void_ptr;
	int test = p->test_number;

	/* EPERM: exs_init has not been called */
	if (test == 1) {
		p->test_value = "EPERM";
		p->test_name = all_mderegister_tests[test - 1];
		setup_fault_catcher(p, 0);
		int ret = exs_mderegister(p->send_mh, 0);
		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EPERM);
	}

	/* EINVAL: addr is null */
	else if (test == 2) {
		p->test_value = "EINVAL";
		p->test_name = all_mderegister_tests[test - 1];
		setup_fault_catcher(p, 0);
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);
		int ret = exs_mderegister(NULL, 1);
		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EINVAL);
	}

	/* EINVAL: EXS_MHANDLE_INVALID */
	else if (test == 3) {
		p->test_value = "EINVAL";
		p->test_name = all_mderegister_tests[test - 1];
		setup_fault_catcher(p, 0);
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);
		int ret = exs_mderegister(EXS_MHANDLE_INVALID, 0);
		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EINVAL);
	}

	/* EINVAL: EXS_MHANDLE_UNREGISTERED */
	else if (test == 4) {
		p->test_value = "EINVAL";
		p->test_name = all_mderegister_tests[test - 1];
		setup_fault_catcher(p, 0);
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);
		int ret = exs_mderegister(EXS_MHANDLE_INVALID, 0);
		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EINVAL);
	}

	/* EINVAL: BAD FLAG */
	else if (test == 5) {
		p->test_value = "EINVAL";
		p->test_name = all_mderegister_tests[test - 1];
		setup_fault_catcher(p, 0);
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);
		int ret = exs_mderegister(p->send_mh, BAD_FLAG);
		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EINVAL);
	}

	/* OK_EXS_MDEREGISTER */
	else if (test == 6) {
		p->test_value = "OK_EXS_MDEREGISTER";
		p->test_name = all_mderegister_tests[test - 1];
		setup_fault_catcher(p, 0);
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);

		if (ok_exs_mregister(p) < 0)
			exit(EXIT_FAILURE);

		int ret = ok_exs_mderegister(p);

		if (ret == 0) {
			print_macro(test, p->test_value, p->test_name, ret,
				    0, 0);
		} else {
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, ENOTSUP);
		}
	}

	/** test number not valid **/
	else if (test != 0) {
		fprintf(stdout, "Invalid Test Number: %d\n", test);
	}

	return NULL;
}

int main(int argc, char *argv[])
{				//arv[1] first arg from test_all.sh
	execute(argc, argv, test_mderegister, all_mderegister_tests, MDEREGISTER_TESTS);
}
