/*
 * Copyright 2007-2017 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#include "result.h"
#include "setup/exs_init_setup.h"
#include "setup/exs_qcreate_setup.h"

#define QCREATE_TESTS 3

static char *all_qcreate_tests[QCREATE_TESTS] = {
	"exs_qcreate(10) [no exs_init]",
	"",
	"exs_qcreate(10)"
};

static void *test_qcreate(void *test_void_ptr)
{
	param *p = (param *) test_void_ptr;
	int test = p->test_number;

	/* EPERM: exs_init has not been called */
	if (test == 1) {
		p->test_value = "EPERM";
		p->test_name = all_qcreate_tests[test - 1];
		setup_fault_catcher(p, 0);

		exs_qhandle_t ret = exs_qcreate(10);

		print_qhandle_macro(test, "EPERM", p->test_name, ret,
				    EPERM);
	}

	/* EINVAL: depth too large */
	else if (test == 2) {

	}

	/* OK_EXS_QCREATE */
	else if (test == 3) {
		p->test_value = "OK_EXS_QCREATE";
		p->test_name = all_qcreate_tests[test - 1];
		setup_fault_catcher(p, 0);

		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);

		ok_exs_qcreate(p);
		print_qhandle_macro(test, p->test_value, p->test_name, p->qh, 0);
	}

	/** test number not valid **/
	else if (test != 0) {
		fprintf(stdout, "Invalid Test Number: %d\n", test);
	}

	return NULL;
}

int main(int argc, char *argv[])
{				//arv[1] first arg from test_all.sh
	execute(argc, argv, test_qcreate, all_qcreate_tests, QCREATE_TESTS);
}
